require "net/smtp"

class Protocols::Smtp
  def initialize
    reset_smtp_client!
  end

  def distribute(mail)
    puts "SEND #{mail.from}\t -> #{mail.to}:\t#{mail.subject}"
    smtp_raw = mail.to_smtp
    $logger.debug smtp_raw.join("=====")
    send_message_safe(*smtp_raw)
  rescue => err
    warn "FAILED #{mail.from}\t -> #{mail.to}:\t#{mail.subject}"
    $logger.debug err
  end

  private def reset_smtp_client!
    @smtp = Net::SMTP.new(
      ENV["SMTP_HOST"], ENV["SMTP_PORT"], tls: ENV["SMTP_TLS"] == "true",
    ).start(
      user: ENV["SMTP_USER"],
      secret: ENV["SMTP_PASSWORD"],
      authtype: :login,
    )
  end

  def send_message_safe(*raw, max_tries: 3)
    if max_tries == 0
      return $logger.error("send_message_safe reached max_tries_limit")
    end

    begin
      @smtp.send_message(*raw)
    rescue EOFError, Net::SMTPServerBusy => err
      warn err.message
      reset_smtp_client!
      send_message_safe(*raw, max_tries: max_tries - 1)
    end
  end
end
